/* Copyright (C) 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package symbols;

import osio.TextBuffer;
import osio.TextPrinter;

import compbase.AccVar;
import compbase.Context;
import compbase.Mthd;
import compbase.Unit;
import compbase.UnitList;

/**
 * MthdSymbols: generate output for method names
 * 
 * @author S. Frenz
 * @version 091001 adopted changed memory interface
 *  version 090207 added copyright notice
 *  version 080701 adopted move of TextBuffer to osio
 *  version 080118 initial version
 */

public class MthdSymbols extends SymbolInformer {
  private TextBuffer myBuffer;
  private int mthdNameParPos;
  
  public static void printValidParameters(TextPrinter v) {
    v.println(NOPARMREQ);
  }
  
  public boolean setParameter(String parm, TextPrinter v) {
    v.println("No parameter expected for MthdSymbols");
    return false;
  }
  
  public boolean generateSymbols(UnitList newUnits, Context ictx) {
    int addedMthdSize, unitNameLen;
    Unit unit;
    Mthd mthds;
    
    ctx=ictx; //required for checkVrbl, access to ictx should be prefered
    myBuffer=new TextBuffer();
    if ((mthdNameParPos=checkVrbl(ictx.rteSMthdBlock, "namePar"))==AccVar.INV_RELOFF) return false;
    addedMthdSize=(4+ctx.arch.allocClearBits)&~ctx.arch.allocClearBits;
    while (newUnits!=null) {
      myBuffer.used=0;
      (unit=newUnits.unit).pack.printFullQID(myBuffer);
      myBuffer.print('.');
      unit.printNameWithOuter(myBuffer);
      myBuffer.print('.');
      unitNameLen=myBuffer.used;
      mthds=unit.mthds;
      while (mthds!=null) {
        if (mthds.outputLocation!=null) {
          ictx.symGenSize+=addedMthdSize;
          myBuffer.used=unitNameLen; //reset to unit name
          mthds.printNamePar(myBuffer);
          ictx.arch.putRef(mthds.outputLocation, mthdNameParPos,
              ictx.allocateString(new String(myBuffer.data, 0, myBuffer.used)), 0);
        }
        mthds=mthds.nextMthd;
      }
      newUnits=newUnits.next;
    }
    return true;
  }
}
